//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PhTriangle.hpp"

#include "MyException.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
PhTriangle::PhTriangle(Group *parent, int phGroupId, const std::string &name, const Vector2D &pt1, const Vector2D &pt2, const Vector2D &pt3) : PhysicObjectLine(parent, phGroupId, name)
{
    m_pts[0] = pt1;
    m_pts[1] = pt2;
    m_pts[2] = pt3;

    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
Vector2D PhTriangle::GetPoint(int nb) const
{
    if(nb < 0 || nb > 2)
    {
        MY_THROW("PhTriangle::GetPoint : Number is not included in [0;2]");
    }

    return m_pts[nb];
}
//------------------------------------------------------------------------------
void PhTriangle::SetPoint(int nb, const Vector2D &pt)
{
    if(nb < 0 || nb > 2)
    {
        MY_THROW("PhTriangle::GetPoint : Number is not included in [0;2]");
    }

    m_pts[nb] = pt;

    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
std::vector<Vector2D> PhTriangle::GetAllPoints() const
{
    float angle = GetGlobalRotation();
    Vector2D pos = GetGlobalPosition();

    std::vector<Vector2D> ptList;
    ptList.resize(3);

    for(size_t i = 0; i < 3; i++)
    {
        ptList[i] = pos + PointRotate(Vector2D(0, 0), m_pts[i], angle);
    }

    return ptList;
}
//------------------------------------------------------------------------------
/*void PhTriangle::RegisterClassLua(lua_State *myLuaState)
{
    luabind::module(myLuaState)
    [
        luabind::class_<Object>("Object")

    ];
}*/
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
} // namespace En
