//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef PHYSICWORLD_HPP
#define PHYSICWORLD_HPP
//------------------------------------------------------------------------------
#include <vector>
#include <map>
#include <utility>

#include "PhysicObject.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
class PhysicWorld
{
public:
    PhysicWorld();
    ~PhysicWorld();

    void Attach(PhysicObject *obj);
    void Detach(PhysicObject *obj);

    void ProceedIntersection();
    void AddPhysicGroup(int phGroupId1, int phGroupId2);

    /** Return the first unused physic group of this physic world */
    int GetUnusedPhGroup() const;

    /** Return the list of objects that collide the given physic group */
    std::vector<CollisionInfo> IsIntersectWithGroup(const PhysicObject &obj, int phGroup) const;


protected:
    void ProceedIntersection(int phGroupId1, int phGroupId2);

    std::map<int, std::vector<PhysicObject*> > m_phTbl;
    std::vector<std::pair<int, int> > m_phGroupTbl;

private:
};
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
#endif // PHYSICWORLD_HPP
