//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include <iostream>
#include <sstream>
#include <fstream>
//------------------------------------------------------------------------------
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
//------------------------------------------------------------------------------
#include "AllObj.hpp"
#include "Global.hpp"
#include "ConfigManager.hpp"

#include "MyException.hpp"

#include <luabind/object.hpp>

#if DEBUG == 1
    #include <SPK.h>
#endif

//------------------------------------------------------------------------------
using namespace std;
using namespace En;
//------------------------------------------------------------------------------
void FailsafeEscapeFn(const std::string &keyName, bool pressed, ButtonEventHandler *obj, const luabind::object &luaFunction)
{
    cout << "Closing..." << endl;
    Global::Quit();
}
//------------------------------------------------------------------------------
/*void FailsafeReturnFn(const std::string &keyName, bool pressed, ButtonEventHandler *obj, const luabind::object &luaFunction)
{
    cout << "Returning to the main menu..." << endl;
    Global::ShowConsole(false);
    Global::LoadMode("Menu");
}*/
//------------------------------------------------------------------------------
int main(int argc, const char* argv[])
{
    #if DEBUG == 1
    cerr << "Count : " << ClassAllocCounter::GetAllocCount() << endl;
    #endif

    try
    {
        // Load the engine
        Global::Engine myEngineInstance(argc, argv);

        try // Engine is on, we can show a text if an error occure
        {
            // Force to render a frame to open a window
            sf::RenderWindow &app = Global::GetApp();
            app.Clear(Color(0, 0, 0));
            app.Display();

            // Search for mode to load...
            std::vector<std::string> argList = Global::GetStdCmdArguments();

            std::string modeToLoad = "Logo";
            for(size_t i = 0; i < argList.size(); i++)
            {
                if(argList[i] == "--mode" && i < argList.size() - 1)
                {
                    modeToLoad = argList[i + 1];
                }
            }

            // Set the mode to load
            Global::LoadMode(modeToLoad);

            // Run the engine
            Global::Run();
        }
        catch(std::exception &e)
        {
            Global::Clear();

            cout << "\n------------- FAILSAFE MODE -------------" << endl;
            cerr << "Unable to load mode !" << endl;
            cerr << "Fatal error:" << endl;
            cerr << e.what() << endl;
            cout << "\nPress Escape to quit." << endl;
            //cout << "Press Enter to return to the main menu." << endl;

            // The console height is maximum
            Global::ShowConsole(true);
            En::Screen *screen = new En::Screen("Debug screen");
            ButtonEventHandler *evt = static_cast<ButtonEventHandler*>(screen->RegisterButtonEvent(luabind::object(), "Escape", true).GetPtr());
            evt->SetEventFunction(&FailsafeEscapeFn);

            /*evt = static_cast<ButtonEventHandler*>(screen->RegisterButtonEvent(luabind::object(), "Return", true));
            evt->SetEventFunction(&FailsafeReturnFn);*/

            Global::ChangeScreen(screen);
            Global::Run();
        }
    }
    catch(std::exception &e)
    {
        cerr << "Fatal error:" << endl;
        cerr << e.what() << endl;
    }

    #if DEBUG == 1
    cerr << "Objects left : " << ClassAllocCounter::GetAllocCount() << endl;
    cerr << "SPK left : " << SPK::SPKFactory::getInstance().getNbObjects() << endl;
    cerr << "ConfigNode left : " << ConfigNode::GetProtectedElementsCount() << endl;
    ConfigNode::PrintLeftProtectedElements();
    #endif

    return 0;
}
